/*
** File: pool.c  
** Project: VxWorks Driver Framework
** Purpose: Pool creation and destruction functions
**
** (C) Copyright Alpha Data 2010
*/

#include "df.h"

/* TO DO - make pool size dynamic */
#define DF_POOL_INITIAL_SIZE (0x100U)
#define DF_POOL_ITEM_ALIGN   (0x10U)

boolean_t
_dfPoolInit(
  DfPool* pPool,
  size_t itemSize)
{
  size_t itemSizeAligned, size;
  void* pBuffer;
  void** pFreeStack;
  unsigned int i;

  dfSpinLockInit(&pPool->system.lock);
  itemSizeAligned = (itemSize + DF_POOL_ITEM_ALIGN - 1) & ~(DF_POOL_ITEM_ALIGN - 1);
  size = itemSizeAligned * DF_POOL_INITIAL_SIZE;
  pBuffer = dfMalloc(size);
  if (NULL == pBuffer) {
    return FALSE;
  }
  pFreeStack = (void**)dfMalloc(sizeof(void*) * DF_POOL_INITIAL_SIZE);
  if (NULL == pFreeStack) {
    dfFree(pBuffer);
    return FALSE;
  }
  pPool->system.stackPtr = 0;
  pPool->system.pBuffer = pBuffer;
  pPool->system.pFreeStack = pFreeStack;
  pPool->system.numItem = DF_POOL_INITIAL_SIZE;
#if DF_DBG_BUILD
  pPool->system.itemSize = itemSize;
  pPool->system.itemSizeAligned = itemSizeAligned;
#endif
  for (i = 0; i < pPool->system.numItem; i++) {
    pFreeStack[i] = (uint8_t*)pBuffer + i * itemSizeAligned;
  }

  return TRUE;
}

void
dfPoolUninit(
  DfPool* pPool)
{
#if DF_DBG_BUILD
  dfAssert(NULL != pPool->system.pBuffer);
  dfAssert(NULL != pPool->system.pFreeStack);
  dfAssert(0 == pPool->system.stackPtr);
#endif
  if (NULL != pPool->system.pBuffer) {
    dfFree(pPool->system.pBuffer);
    pPool->system.pBuffer = NULL;
  }
  if (NULL != pPool->system.pFreeStack) {
    dfFree(pPool->system.pFreeStack);
    pPool->system.pFreeStack = NULL;
  }
}
