/*
** File: sempool.c  
** Project: VxWorks Driver Framework
** Purpose: Semaphore pool
**
** (C) Copyright Alpha Data 2010
*/

#include "df.h"
#include "dfvxworks.h"

#if DFVXW_EVENT_USE_SEMPOOL

# define DFVXW_SEMPOOL_INITIAL_SIZE (0x100)

SEM_ID g_dfVxwSemPoolArray[DFVXW_SEMPOOL_INITIAL_SIZE];
unsigned int g_dfVxwSemPoolPtr = 0;
unsigned int g_dfVxwSemPoolSize = 0;
DfSpinLock g_dfVxwSemPoolLock;

#endif

boolean_t
dfVxwSemPoolInit(
  void)
{
#if DFVXW_EVENT_USE_SEMPOOL
  unsigned int i;

  g_dfVxwSemPoolSize = DFVXW_SEMPOOL_INITIAL_SIZE;
  for (i = 0; i < g_dfVxwSemPoolSize; i++) {
    g_dfVxwSemPoolArray[i] = NULL;
  }
  g_dfVxwSemPoolPtr = 0;
  dfSpinLockInit(&g_dfVxwSemPoolLock);

  for (i = 0; i < g_dfVxwSemPoolSize; i++) {
    g_dfVxwSemPoolArray[i] = semBCreate(SEM_Q_FIFO, SEM_EMPTY);
    if (g_dfVxwSemPoolArray[i] == NULL) {
      dfVxwSemPoolUninit();
      return FALSE;
    }
  }
#endif

  return TRUE;
}

void
dfVxwSemPoolUninit(
  void)
{
#if DFVXW_EVENT_USE_SEMPOOL
  unsigned int i;

  for (i = 0; i < g_dfVxwSemPoolSize; i++) {
    if (g_dfVxwSemPoolArray[i] != NULL) {
      semDelete(g_dfVxwSemPoolArray[i]);
      g_dfVxwSemPoolArray[i] = NULL;
    }
  }
  if (g_dfVxwSemPoolPtr) {
    dfDebugPrint(0, ("dfVxwSemPoolUninit: some semaphores not returned to pool, semPoolPtr=%d\n", g_dfVxwSemPoolPtr));
  }
  g_dfVxwSemPoolPtr = 0;
#endif
}
